require 'spec_helper'

describe DotHelper do
  describe "with example Agents" do
    class Agents::DotFoo < Agent
      default_schedule "2pm"

      def check
        create_event :payload => {}
      end
    end

    class Agents::DotBar < Agent
      cannot_be_scheduled!

      def check
        create_event :payload => {}
      end
    end

    before do
      stub(Agents::DotFoo).valid_type?("Agents::DotFoo") { true }
      stub(Agents::DotBar).valid_type?("Agents::DotBar") { true }
    end

    describe "#agents_dot" do
      before do
        @agents = [
          @foo = Agents::DotFoo.new(name: "foo").tap { |agent|
            agent.user = users(:bob)
            agent.save!
          },

          @bar1 = Agents::DotBar.new(name: "bar1").tap { |agent|
            agent.user = users(:bob)
            agent.sources << @foo
            agent.save!
          },

          @bar2 = Agents::DotBar.new(name: "bar2").tap { |agent|
            agent.user = users(:bob)
            agent.sources << @foo
            agent.propagate_immediately = true
            agent.disabled = true
            agent.save!
          },

          @bar3 = Agents::DotBar.new(name: "bar3").tap { |agent|
            agent.user = users(:bob)
            agent.sources << @bar2
            agent.save!
          },
        ]
      end

      it "generates a DOT script" do
        agents_dot(@agents).should =~ %r{
          \A
          digraph \s foo \{
            node \[ [^\]]+ \];
            (?<foo>\w+) \[label=foo\];
            \k<foo> -> (?<bar1>\w+) \[style=dashed\];
            \k<foo> -> (?<bar2>\w+) \[color="\#999999"\];
            \k<bar1> \[label=bar1\];
            \k<bar2> \[label="bar2 \s \(Disabled\)",style="rounded,dashed",color="\#999999",fontcolor="\#999999"\];
            \k<bar2> -> (?<bar3>\w+) \[style=dashed,color="\#999999"\];
            \k<bar3> \[label=bar3\];
          \}
          \z
        }x
      end

      it "generates a richer DOT script" do
        agents_dot(@agents, true).should =~ %r{
          \A
          digraph \s foo \{
            node \[ [^\]]+ \];
            (?<foo>\w+) \[label=foo,URL="#{Regexp.quote(agent_path(@foo))}"\];
            \k<foo> -> (?<bar1>\w+) \[style=dashed\];
            \k<foo> -> (?<bar2>\w+) \[color="\#999999"\];
            \k<bar1> \[label=bar1,URL="#{Regexp.quote(agent_path(@bar1))}"\];
            \k<bar2> \[label="bar2 \s \(Disabled\)",URL="#{Regexp.quote(agent_path(@bar2))}",style="rounded,dashed",color="\#999999",fontcolor="\#999999"\];
            \k<bar2> -> (?<bar3>\w+) \[style=dashed,color="\#999999"\];
            \k<bar3> \[label=bar3,URL="#{Regexp.quote(agent_path(@bar3))}"\];
          \}
          \z
        }x
      end
    end
  end

  describe DotHelper::DotDrawer do
    describe "#id" do
      it "properly escapes double quotaion and backslash" do
        DotHelper::DotDrawer.draw(foo: "") {
          id('hello\\"')
        }.should == '"hello\\\\\\""'
      end
    end
  end
end
